__author__="Simon"
__date__ ="$31.05.2013 17:05:27$"

import pygame
import pygame.sprite
import math
from pygame.locals import *

class Vector(object):
    def __init__(self, x=0.0, y=0.0):
        self.x = x
        self.y = y

    def __str__(self):
        return "(%s, %s)"%(self.x, self.y)

    def get_magnitude(self):
        return math.sqrt(self.x**2 + self.y**2)

    def normalize(self):
        magnitude = self.get_magnitude()
        self.x /= magnitude
        self.y /= magnitude

    @classmethod
    def vector_from_points(cls,from_p,to_p):
        return cls(to_p[0]-from_p[0],to_p[1]-from_p[1])

class Spielfigur(pygame.sprite.Sprite):
    def __init__(self, color, initial_position, heading):
        pygame.sprite.Sprite.__init__(self) # call the superclass constructor
        self.image = pygame.Surface((20,20)) # define the image surface
        self.image.fill(color)
        self.rect = self.image.get_rect() # define the rect
        self.rect.topleft = initial_position
        self.speed = 0
        self.heading = heading

    def update(self, time_passed):
        moved_distance = time_passed*self.speed
        if self.heading != None:
            new_centerx = self.rect.centerx + moved_distance*self.heading.x
            if new_centerx > 9 and new_centerx < 591:
                self.rect.centerx = new_centerx

class Gegner(pygame.sprite.Sprite):
#        self.image = pygame.Surface((40,40)) # define the image surface
    def __init__(self, color, initial_position):
        pygame.sprite.Sprite.__init__(self) # call the superclass constructor
        self.image = pygame.Surface((40, 40), pygame.SRCALPHA, 32)
        pygame.draw.circle(self.image, color, (20, 20), 20, 0)
        self.rect = self.image.get_rect() # define the rect
        self.rect.topleft = initial_position
        self.speed = 200
        self.heading = Vector.vector_from_points((0,0),(1,0))
        self.direction = "right"

    def update(self, time_passed):
        moved_distance = math.ceil(time_passed * self.speed)
        if self.direction == "right":
            self.rect.x += moved_distance*self.heading.x
        if self.direction == "left":
            self.rect.x -= moved_distance*self.heading.x
        if (self.rect.x > 560) and (self.direction == "right"):
            self.direction = "left"
            self.rect.y += 50
        if (self.rect.x < 0) and (self.direction == "left"):
            self.direction = "right"
            self.rect.y += 50

class Bombe(pygame.sprite.Sprite):
    def __init__(self, pos):
        pygame.sprite.Sprite.__init__(self) # call the superclass constructor
        self.image = pygame.Surface((16, 16), pygame.SRCALPHA, 32)
        pygame.draw.circle(self.image, pygame.Color(255, 255, 255), (5, 5), 5, 0)
        self.rect = self.image.get_rect()
        self.speed = 300
        self.rect.center = pos
        self.heading = Vector.vector_from_points((0,0),(0,1))
        self.heading.normalize()

    def update(self, time_passed):
        moved_distance = math.ceil(time_passed * self.speed)
        self.rect.y -= moved_distance*self.heading.y
        if self.rect.y < 0:
            self.kill()

class Anzeige(pygame.sprite.Sprite):
    def __init__(self, pos):
        pygame.sprite.Sprite.__init__(self) # call the superclass constructor
        self.font = pygame.font.Font(None, 50)
        self.color = (255, 255, 255)
        self.counter = 0
        self.image = self.font.render("%d"%(self.counter), True, self.color)
        self.rect = pos

    def update(self):
        self.counter += 1
        self.image = self.font.render("%d"%(self.counter), True, self.color)


pygame.init()

screen = pygame.display.set_mode((600, 500), 0, 32)
heading = Vector.vector_from_points((0,0),(1,0))
heading.normalize()

spielfigur = Spielfigur((255,255,255),(280,480),None)

gegner_sprites = ([(255,0,0),(0,0)],[(0,255,0),(80,0)],[(0,0,255),(160,0)],[(255,255,0),(240,0)],[(255,0,255),(320,0)])
gegner = pygame.sprite.Group()
for g in gegner_sprites:
    gegner.add(Gegner(g[0],g[1]))

bomben = pygame.sprite.Group()

anzeige = Anzeige((560,0))
anzeige2 = Anzeige((540,0))

sprites = pygame.sprite.RenderUpdates()
sprites.add(anzeige)

clock = pygame.time.Clock()

while True:
    for event in pygame.event.get():
        if event.type == QUIT:
            exit()

        pressed = pygame.key.get_pressed() # pressed buttons

        if (pressed[K_LEFT] == 0) and (pressed[K_RIGHT] == 0):
            spielfigur.speed = 0
            spielfigur.heading = None
        if pressed[K_LEFT]:
            spielfigur.speed = 400
            heading = Vector.vector_from_points((1,0),(0,0))
            heading.normalize()
            spielfigur.heading = heading
        if pressed[K_RIGHT]:
            spielfigur.speed = 400
            heading = Vector.vector_from_points((0,0),(1,0))
            heading.normalize()
            spielfigur.heading = heading
        if pressed[K_SPACE]:
            bombe = Bombe((spielfigur.rect.centerx,spielfigur.rect.centery))
            bomben.add(bombe)

    for b in bomben:
        for g in gegner:
            if pygame.sprite.collide_rect(g,b):
                gegner.remove(g)
                anzeige.update()

    if anzeige.counter == 5:
        print("YOU WIN")

    screen.fill((0,0,0))
    time_passed = clock.tick() / 1000.0

    spielfigur.update(time_passed)
    gegner.update(time_passed)
    bomben.update(time_passed)

    screen.blit(spielfigur.image,spielfigur.rect)
    gegner.draw(screen)
    bomben.draw(screen)
    screen.blit(anzeige.image,anzeige.rect)
    screen.blit(anzeige2.image,anzeige2.rect)

    pygame.display.update()
